class Scene {
    constructor(canvas) {
        this.canvas = canvas;
        this.context = canvas.getContext('2d');
        this.objects = [];
        this.platforms = [];
        this.maxHeight = 0;
    }

    width() {
        return this.canvas.width;
    }

    height() {
        return this.canvas.height;
    }

    resize(width, height) {
        this.canvas.width = width;
        this.canvas.height = height;
    }

    autoResize() {
        this.canvas.width = window.innerWidth;
        this.canvas.height = window.innerHeight;
    }

    clear() {
        this.context.clearRect(0, 0, this.width(), this.height());
    }

    drawBackground() {
        this.context.fillStyle = 'rgb(255, 0, 0)';
        this.context.beginPath();
        this.context.rect(0, 0, this.canvas.width, this.canvas.height);
        this.context.fill();
    }

    update(time = 0.002) {
        for (let object of this.objects) {

            // Update high score
            this.maxHeight = Math.max(object.y, this.maxHeight);

            let previous = {
                x: object.x,
                y: object.y + object.size / 2,
            };

            object.update(time);

            let next = {
                x: object.x,
                y: object.y + object.size / 2,
                size: object.size,
            };

            // Detect collision with platform
            for (let platform of this.platforms) {

                let p = {
                    x: platform.x,
                    y: platform.y + platform.height / 2,
                    width: platform.width,
                    height: platform.height,
                };

                if (previous.y >= p.y && next.y < p.y) {
                    if (next.x >= p.x - p.width / 2 - next.size / 2) {
                        if (next.x <= p.x + p.width / 2 + next.size / 2) {
                            // Collision detected
                            object.collide(p.y - next.size/2);
                        }
                    }
                }
            }

            // Collision with the ground
            if (object.y <= 0) {
                object.collide(0);
            }

            // Collisions with the border of the screen
            object.x = Math.max(object.x, object.size / 2);
            object.x = Math.min(object.x, 1 - object.size / 2);
        }
    }

    addObject(object) {
        this.objects.push(object);
    }

    addPlatform(object) {
        this.platforms.push(object);
    }

    render() {
        this.clear();
        this.drawBackground();

        for (let object of this.objects) {
            this.drawObject(object);
        }

        for (let platform of this.platforms) {
            this.drawPlatform(platform);
        }

        this.drawHud();
    }

    drawObject(object) {
        let size = object.size * this.width();

        this.context.fillStyle = 'rgb(0, 0, 0)';
        this.context.beginPath();

        this.context.rect(
            (object.x - object.size / 2) * this.width(),
            (1 - object.y - object.size / 2) * this.height(),
            size,
            size,
        );
        this.context.fill();
    }

    drawPlatform(object) {
        let width = object.width * this.width();
        let height = object.height * this.height();

        this.context.fillStyle = object.color || 'rgb(0, 0, 0)';
        this.context.beginPath();

        this.context.rect(
            (object.x - object.width / 2) * this.width(),
            (1 - object.y - object.height / 2) * this.height(),
            width,
            height,
        );
        this.context.fill();
    }

    drawHud() {
        this.context.font = "20px Arial";
        this.context.fillStyle = 'rgb(255, 255, 255)';
        this.context.fillText("High score: " + Math.floor(100 * this.maxHeight), 0, 20);
    }

}
